package dk.nsp.idsas.dao;

import dk.nsp.idsas.util.ErrorMessages;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import javax.sql.DataSource;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;

public class SaltDaoTest extends AbstractDaoTest {
    @Autowired
    DataSource dataSource;

    private SaltDao saltDao;

    @Before
    public void setUp() throws Exception {
        saltDao = new SaltDao(dataSource);
    }

    @Test
    public void getLatestSalt_WhenSaltTableIsEmpty_ThrowsException() {
        RuntimeException e = assertThrows(RuntimeException.class, () -> saltDao.getLatestSalt());

        assertEquals(ErrorMessages.noSaltError(), e.getMessage());
    }

    @Test
    public void getLatestSalt_AfterInsert_ReturnsInsertedSalt() {
        String salt = "test salt";
        saltDao.insert(salt);

        String latestSalt = saltDao.getLatestSalt();

        assertEquals(salt, latestSalt);
    }

    @Test
    public void getLatestSalt_AfterMultipleInserts_ReturnsLatestSalt() {
        saltDao.insert("salt1");
        saltDao.insert("salt2");
        saltDao.insert("salt3");
        saltDao.insert("salt4");

        String latestSalt = saltDao.getLatestSalt();

        assertEquals("salt4", latestSalt);
    }
}