package dk.nsp.idsas.util.health;

import dk.nsp.idsas.dao.SaltDao;
import org.apache.commons.httpclient.HttpStatus;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class SaltHealthIndicatorTest {

    private SaltDao saltDao;
    private SaltHealthIndicator saltHealthIndicator;

    @Before
    public void setUp() throws Exception {
        saltDao = Mockito.mock(SaltDao.class);
        saltHealthIndicator = new SaltHealthIndicator(saltDao);
    }

    @Test
    public void getHealthInfo_WhenSaltDaoDoesNotFail_ReturnsHealthIndicationOK() {
        HealthIndication healthInfo = saltHealthIndicator.getHealthInfo();

        assertEquals(HttpStatus.SC_OK, healthInfo.getStatus());
        assertTrue(healthInfo.getText().contains("OK"));
    }

    @Test
    public void getHealthInfo_WhenSaltDaoFails_ReturnsHealthIndicationUnavailable() {
        Mockito.doThrow(RuntimeException.class).when(saltDao).getLatestSalt();

        HealthIndication healthInfo = saltHealthIndicator.getHealthInfo();

        assertEquals(HttpStatus.SC_SERVICE_UNAVAILABLE, healthInfo.getStatus());
        assertTrue(healthInfo.getText().contains("Unavailable"));
    }
}