package dk.nsp.idsas.test.stepdefinitions;

import dk.nsp.idsas.test.idptest.SOAPOperation;
import dk.nsp.idsas.test.idptest.ServiceCaller;
import dk.nsp.test.idp.builder.Builders;
import dk.nsp.test.idp.model.EmployeeIdentity;
import dk.nsp.test.idp.vault.RemoteCredentialVault;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.CreateBlurringRequestType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.PatientIdClassificationType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.PatientIdType;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;
import org.springframework.beans.factory.annotation.Value;

import javax.xml.bind.JAXBException;
import javax.xml.datatype.DatatypeConfigurationException;
import java.net.URI;
import java.net.URISyntaxException;
import java.time.LocalDateTime;
import java.util.UUID;

import static dk.nsp.idsas.test.util.Util.toXmlGregorian;
import static dk.nsp.test.idp.builder.Builders.*;
import static org.junit.Assert.assertTrue;

public class AuthStepDefinitions {
    @Value("${idsas.service.endpoint}" + "/20230601/service")
    private String url;

    private final ServiceCaller serviceCaller = new ServiceCaller();
    private String fejlbesked = "";

    @When("Jeg kalder med CVR {string}, der ikke er white listed")
    public void jegKalderMedCVRDerIkkeErWhiteListed(String cvr) throws URISyntaxException, JAXBException, DatatypeConfigurationException {
        CreateBlurringRequestType request = new CreateBlurringRequestType();
        PatientIdType patientId = new PatientIdType();
        patientId.setId("0101000000");
        patientId.setClassification(PatientIdClassificationType.CPR);
        request.setPatientId(patientId);
        request.setEndDateTime(toXmlGregorian(LocalDateTime.now()));
        try {
            serviceCaller.callRoot(peterNaur(cvr),
                    request, SOAPOperation.CreateBlurring, new URI(url),
                    CreateBlurringRequestType.class,
                    String.class);
        } catch (RuntimeException e) {
            fejlbesked = e.getMessage();
        }
    }

    @Then("Jeg får en SOAP fault tilbage som indeholder CVR {string} og teksten {string}")
    public void jegFårEnSOAPFaultTilbageSomIndeholderCVROgTeksten(String cvr, String tekst) {
        System.out.println(fejlbesked);
        assertTrue(fejlbesked.contains(cvr));
        assertTrue(fejlbesked.contains(tekst));
    }

    public static EmployeeIdentity peterNaur(String cvr) {
        return Builders.employeeIdentity()
                .representing(employee()
                        .identifiedBy(UUID.fromString("4a4c9843-58b5-4694-95d0-ea5ab80823da")))
                .employedAt(organization()
                        .named("Sundhedsdatastyrelsen")
                        .identifiedBy(cvr))
                .usedBy(serviceConsumer()
                        .named("Service Consumer Test")
                        .identifiedBy(new RemoteCredentialVault(URI.create("https://www.nspop.dk/download/attachments/190481050/NSP_Test_Service_Consumer_sds.p12"), "Test1234".toCharArray(), true)))
                .verifiedBy(identityProvider()
                        .named("https://idp.test.nspop.dk")
                        .identifiedBy(new RemoteCredentialVault(URI.create("https://www.nspop.dk/download/attachments/190481050/NSP_Test_Identity_Provider_sds.p12"), "Test1234".toCharArray(), true)))
                .using(securityTokenService()
                        .at(URI.create("http://test1.ekstern-test.nspop.dk:8080/sts/services/BST2SOSI")))
                .build();
    }


}
