package dk.nsp.idsas.test.stepdefinitions;

import dk.nsp.idsas.test.idptest.SOAPOperation;
import dk.nsp.idsas.test.idptest.ServiceCaller;
import dk.nsp.idsas.test.util.TestIdentities;
import dk.nsp.test.idp.OrganizationIdentities;
import dk.nsp.test.idp.model.Identity;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.GetCurrentSaltRequest;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;
import org.springframework.beans.factory.annotation.Value;

import java.net.URI;

import static org.junit.Assert.*;


public class SaltStepDefinitions {

    @Value("${idsas.service.endpoint}" + "/20230601/service")
    private String serviceEndpoint;

    private final ServiceCaller serviceCaller = new ServiceCaller();

    private String response = null;
    private boolean requestFailed = false;

    @When("en {string} forespørger salt")
    public void enXForespørgerSalt(String userType) {
        Identity identity = TestIdentities.identityFor(userType);
        sendGetCurrentSaltRequest(identity);
    }

    @When("en ikke whitelisted systembruger forespørger salt")
    public void enIkkeWhitelistedSystembrugerForespørgerSalt() {
        Identity identity = OrganizationIdentities.testOrganisation98021838();
        sendGetCurrentSaltRequest(identity);
    }

    private void sendGetCurrentSaltRequest(Identity identity) {
        GetCurrentSaltRequest request = new GetCurrentSaltRequest();
        try {
            response = serviceCaller.callRoot(identity,
                    request, SOAPOperation.GetCurrentSalt, new URI(serviceEndpoint),
                    GetCurrentSaltRequest.class,
                    String.class);
        } catch (Exception e) {
            response = e.getMessage();
            requestFailed = true;
        }
    }

    @Then("returneres et salt")
    public void returneresEtSalt() {
        assertFalse(response, requestFailed);
        assertNotNull(response);
    }

    @Then("fejler forespørgslen på salt")
    public void fejlerForespørgslen() {
        assertTrue(requestFailed);
    }
}