package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;

import java.util.List;
import java.util.stream.Collectors;

public class LookupService {
    private BlurringDao blurringDao;
    private AuditLogger auditLogger;
    private String allowedClientSystemName;

    public LookupService(BlurringDao blurringDao, AuditLogger auditLogger, String allowedClientSystemName) {
        this.blurringDao = blurringDao;
        this.auditLogger = auditLogger;
        this.allowedClientSystemName = allowedClientSystemName;
    }

    public List<String> getActiveBlurrings(IdsasActor actor, String patientId, String patientIdType) throws IdsasServiceException {
        // Validering af, at actor er en systembruger
        if (!actor.getActorType().equals(IdsasActor.IdsasActorType.SYSTEM)) {
            throw new IdsasServiceException(ErrorMessages.userTypeNotSupported(actor.getActorType().name()), true);
        }

        // Validering af, at systembrugeren har et clientName med den korrekte værdi
        if (!actor.getClientSystemName().equals(allowedClientSystemName)) {
            throw new IdsasServiceException(ErrorMessages.clientSystemNameError(actor.getClientSystemName(), allowedClientSystemName), true);
        }

        // Hente entiteter
        List<BlurringEntity> entityList = blurringDao.getActiveBlurrings(patientId, patientIdType);

        // Mappe entiteter til liste af id'ere
        List<String> orgIdList = entityList.stream().map(BlurringEntity::getOrganisationId).collect(Collectors.toList());

        // Audit logning
        auditLogger.logGetBlurredOrganisations(patientId);

        return orgIdList;
    }
}

