package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.AlivenessDao;
import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.security.IdsasActorProvider;
import dk.nsp.idsas.security.impl.IdsasActorProviderImpl;
import dk.nsp.idsas.util.AuditLogger;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;

import javax.sql.DataSource;

@Configuration
@PropertySources({
        @PropertySource(value = "idsas-lookup.properties", encoding = "ISO-8859-1")
})
public class ServiceSetup {
    public static final String ANNOTATION_CONFIG_CONTEXT = "ANNOTATION_CONFIG_APPLICATION_CONTEXT";
    private static final Logger logger = Logger.getLogger(ServiceSetup.class);

    @Value("${idsas.datasource.jndi}")
    private String dataSourceName;

    @Value("${idsas.allowed.client.system.name:http://system.nspop.dk/sts}")
    private String allowedClientSystemName;

    @Bean
    public DataSource dataSource() {
        return new JndiDataSourceLookup().getDataSource(dataSourceName);
    }

    @Bean
    public AlivenessDao alivenessDao(DataSource dataSource) {
        return new AlivenessDao(dataSource);
    }

    @Bean WhitelistDao whitelistDao(DataSource dataSource) {
        return new WhitelistDao(dataSource);
    }

    @Bean
    public BlurringDao blurringDao(DataSource dataSource) {
        return new BlurringDao(dataSource);
    }

    @Bean
    public SaltDao saltDao(DataSource dataSource) {
        return new SaltDao(dataSource);
    }

    @Bean
    public IdsasActorProvider idsasActorProvider() {
        return new IdsasActorProviderImpl();
    }

    @Bean
    public LookupService lookupService(BlurringDao blurringDao, AuditLogger auditLogger, String allowedClientSystemName) {
        return new LookupService(blurringDao, auditLogger, allowedClientSystemName);
    }

    @Bean
    public AuditLogger auditLogger() {
        return new AuditLogger();
    }

    @Bean(name="allowedClientSystemName")
    public String getAllowedClientSystemName() {
        return allowedClientSystemName;
    }
}
