package dk.nsp.idsas.ws._20230601;

import java.util.List;
import java.util.stream.Collectors;
import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;

import dk.nsp.idsas.security.IdsasActorProvider;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.service.IdsasServiceException;
import dk.nsp.idsas.service.LookupService;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.BlurredOrganisationsResponseType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.GetBlurredOrganisationsRequestType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.IdsasLookupServicePortType;

import org.apache.cxf.annotations.SchemaValidation;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.*;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name = "IdsasLookupService", urlPatterns = {"/20230601/lookup"})
@WebService(serviceName = "IdsasLookupService", wsdlLocation = "wsdl/20230601/idsas-lookup.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/",
        portName = "IdsasLookupServicePort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
public class IdsasLookupServiceWS extends IdsasAbstractWS implements IdsasLookupServicePortType {
    private static final Logger logger = Logger.getLogger(IdsasLookupServiceWS.class);

    @Autowired
    private IdsasActorProvider idsasActorProvider;

    @Autowired
    private LookupService lookupService;

    @Override
    public BlurredOrganisationsResponseType getBlurredOrganisations(GetBlurredOrganisationsRequestType getBlurredOrganisationsRequest) {
        String patientCpr = getBlurredOrganisationsRequest.getPatientId().getId();
        logger.trace(String.format("Enter get blurred organisations for patient %s", patientCpr));
        String patientIdType = getBlurredOrganisationsRequest.getPatientId().getClassification().name();

        // Liste af blurrings hentes
        BlurredOrganisationsResponseType entityList = new BlurredOrganisationsResponseType();
        try {
            IdsasActor actor = idsasActorProvider.getActor();
            List<String> orgs = lookupService.getActiveBlurrings(actor, patientCpr, patientIdType);
            entityList.getOrganisationIds().addAll(orgs.stream().map(IdsasLookupServiceWS::mapToOrganisationId).collect(Collectors.toList()));
            return entityList;

        } catch (IdsasServiceException e) {
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage());
        } catch (Exception e) {
            logger.error("Failed to get blurred organisations", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error");
        }
    }

    private static OrganisationIdType mapToOrganisationId(String org) {
        OrganisationIdType orgId = new OrganisationIdType();
        orgId.setId(org);
        orgId.setClassification(OrganisationIdClassificationType.CVR);
        return orgId;
    }
}
