package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.WhitelistType;

public class SaltService {
    private final SaltDao saltDao;
    private final WhitelistDao whitelistDao;

    public SaltService(SaltDao saltDao, WhitelistDao whitelistDao) {
        this.saltDao = saltDao;
        this.whitelistDao = whitelistDao;
    }

    public String getCurrentSalt(IdsasActor actor) throws IdsasServiceException {
        validateSystemUser(actor.getActorType());
        validateClientWhitelisted(actor.getClientKey());
        return saltDao.getLatestSalt();
    }

    private void validateSystemUser(IdsasActor.IdsasActorType actorType) throws IdsasServiceException {
        if (!actorType.equals(IdsasActor.IdsasActorType.SYSTEM)) {
            throw new IdsasServiceException(ErrorMessages.userTypeNotSupported(actorType.name()), true);
        }
    }

    private void validateClientWhitelisted(String clientKey) throws IdsasServiceException {
        if (!whitelistDao.isWhitelisted(clientKey, WhitelistType.SALT)) {
            throw new IdsasServiceException(ErrorMessages.clientKeyWhitelistError(clientKey, WhitelistType.SALT), true);
        }
    }
}
