package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.AlivenessDao;
import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.http.PoolingHttpClientFactory;
import dk.nsp.idsas.security.IdsasActorProvider;
import dk.nsp.idsas.security.impl.IdsasActorProviderImpl;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.integration.ServiceErrorCallback;
import dk.nsp.idsas.integration.personinformation.PersonInformationClient;
import dk.nsp.idsas.integration.sores.SoresClient;

import org.apache.http.client.HttpClient;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;

import javax.sql.DataSource;
import java.time.Duration;

@Configuration
@PropertySources({
        @PropertySource(value = "idsas.properties", encoding = "ISO-8859-1")
})
public class ServiceSetup {
    public static final String ANNOTATION_CONFIG_CONTEXT = "ANNOTATION_CONFIG_APPLICATION_CONTEXT";
    private static final Logger logger = Logger.getLogger(ServiceSetup.class);

    @Value("${idsas.datasource.jndi}")
    private String dataSourceName;

    @Value("${personinformation.url}")
    private String personInformationUrl;

    @Value("${personinformation.errorcount.duration:PT10M}")
    private String personInformationErrorCountDuration;

    @Value("${sores.url}")
    private String soresUrl;

    @Value("${sores.errorcount.duration:PT10M}")
    private String soresErrorCountDuration;

    @Bean
    public DataSource dataSource() {
        return new JndiDataSourceLookup().getDataSource(dataSourceName);
    }

    @Bean
    public PoolingHttpClientFactory poolingHttpClientFactory() {
        return new PoolingHttpClientFactory();
    }

    @Bean
    public AuditLogger auditLogger() {
        return new AuditLogger();
    }

    @Bean
    public IdsasActorProvider idsasActorProvider() {
        return new IdsasActorProviderImpl();
    }

    @Bean
    public AlivenessDao alivenessDao(DataSource dataSource) {
        return new AlivenessDao(dataSource);
    }

    @Bean
    WhitelistDao whitelistDao(DataSource dataSource) {
        return new WhitelistDao(dataSource);
    }

    @Bean
    public BlurringDao blurringDao(DataSource dataSource) {
        return new BlurringDao(dataSource);
    }

    @Bean
    public SaltDao saltDao(DataSource dataSource) {
        return new SaltDao(dataSource);
    }

    @Bean
    public BlurringService blurringService(WhitelistDao whitelistDao, BlurringDao blurringDao, AuditLogger auditLogger, PersonInformationClient personInformationClient, SoresClient soresClient) {
        return new BlurringService(whitelistDao, blurringDao, auditLogger, personInformationClient, soresClient);
    }

    @Bean
    public SaltService saltService(SaltDao saltDao, WhitelistDao whitelistDao) {
        return new SaltService(saltDao, whitelistDao);
    }

    @Bean
    public PersonInformationClient personInformationClient(PoolingHttpClientFactory poolingHttpClientFactory) {
        HttpClient httpClient = poolingHttpClientFactory.getClient();
        return new PersonInformationClient(httpClient, personInformationUrl, Duration.parse(personInformationErrorCountDuration));
    }

    @Bean
    public SoresClient soresClient(PoolingHttpClientFactory poolingHttpClientFactory) {
        HttpClient httpClient = poolingHttpClientFactory.getClient();
        return new SoresClient(httpClient, soresUrl,
                new ServiceErrorCallback(Duration.parse(soresErrorCountDuration)));
    }

}
