package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.WhitelistType;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;

public class SaltServiceTest {

    private SaltDao saltDao;
    private WhitelistDao whitelistDao;
    private SaltService saltService;

    @Before
    public void setUp() throws Exception {
        saltDao = Mockito.mock(SaltDao.class);
        whitelistDao = Mockito.mock(WhitelistDao.class);
        saltService = new SaltService(saltDao, whitelistDao);
    }

    @Test
    public void getCurrentSalt_WithNonSystemUser_ThrowsException() {
        IdsasActor sundhedsFagligActor = IdsasActor.createSundhedsFaglig("");

        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> saltService.getCurrentSalt(sundhedsFagligActor));

        assertEquals(ErrorMessages.userTypeNotSupported(sundhedsFagligActor.getActorType().name()), e.getMessage());
    }

    @Test
    public void getCurrentSalt_WhereClientKeyIsNotWhitelisted_ThrowsException() {
        IdsasActor systemActor = IdsasActor.createSystem("", "", "testClientKey");
        Mockito.when(whitelistDao.isWhitelisted(systemActor.getClientKey(), WhitelistType.SALT)).thenReturn(false);

        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> saltService.getCurrentSalt(systemActor));

        assertEquals(ErrorMessages.clientKeyWhitelistError(systemActor.getClientKey(), WhitelistType.SALT), e.getMessage());
    }

    @Test
    public void getCurrentSalt_WithWhitelistedSystemActor_ReturnsSalt() throws IdsasServiceException {
        IdsasActor systemActor = IdsasActor.createSystem("", "", "testClientKey");
        Mockito.when(whitelistDao.isWhitelisted(systemActor.getClientKey(), WhitelistType.SALT)).thenReturn(true);
        String testSalt = "test salt";
        Mockito.when(saltDao.getLatestSalt()).thenReturn(testSalt);

        String currentSalt = saltService.getCurrentSalt(systemActor);

        assertEquals(testSalt, currentSalt);
    }
}